import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/dropdown.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/models/accuracy.dart';
import 'package:diameter/models/meal.dart';
import 'package:diameter/models/meal_category.dart';
import 'package:diameter/models/meal_portion_type.dart';
import 'package:diameter/models/meal_source.dart';
import 'package:diameter/models/settings.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/utils/utils.dart';
import 'package:flutter/material.dart';

class MealDetailScreen extends StatefulWidget {
  static const String routeName = '/meal';
  final int id;

  const MealDetailScreen({Key? key, this.id = 0}) : super(key: key);

  @override
  _MealDetailScreenState createState() => _MealDetailScreenState();
}

class _MealDetailScreenState extends State<MealDetailScreen> {
  Meal? _meal;
  bool _isNew = true;
  bool _isSaving = false;

  final GlobalKey<FormState> _mealForm = GlobalKey<FormState>();

  final _valueController = TextEditingController(text: '');
  final _carbsRatioController = TextEditingController(text: '');
  final _portionSizeController = TextEditingController(text: '');
  final _carbsPerPortionController = TextEditingController(text: '');
  final _delayedBolusRateController = TextEditingController(text: '');
  final _delayedBolusDurationController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');

  MealSource? _mealSource;
  MealCategory? _mealCategory;
  MealPortionType? _mealPortionType;
  Accuracy? _portionSizeAccuracy;
  Accuracy? _carbsRatioAccuracy;

  List<MealCategory> _mealCategories = [];
  List<MealPortionType> _mealPortionTypes = [];
  List<MealSource> _mealSources = [];
  List<Accuracy> _portionSizeAccuracies = [];
  List<Accuracy> _carbsRatioAccuracies = [];

  @override
  void initState() {
    super.initState();

    reload();

    _portionSizeAccuracies = Accuracy.getAllForPortionSize();
    _carbsRatioAccuracies = Accuracy.getAllForCarbsRatio();
    _mealCategories = MealCategory.getAll();
    _mealPortionTypes = MealPortionType.getAll();
    _mealSources = MealSource.getAll();

    if (_meal != null) {
      _valueController.text = _meal!.value;
      _carbsRatioController.text = (_meal!.carbsRatio ?? '').toString();
      _portionSizeController.text = (_meal!.portionSize ?? '').toString();
      _carbsPerPortionController.text =
          (_meal!.carbsPerPortion ?? '').toString();
      _delayedBolusRateController.text =
          (_meal!.delayedBolusRate ?? '').toString();
      _delayedBolusDurationController.text =
          (_meal!.delayedBolusDuration ?? '').toString();
      _notesController.text = _meal!.notes ?? '';

      _mealSource = _meal!.mealSource.target;
      _mealCategory = _meal!.mealCategory.target;
      _mealPortionType = _meal!.mealPortionType.target;
      _portionSizeAccuracy = _meal!.portionSizeAccuracy.target;
      _carbsRatioAccuracy = _meal!.carbsRatioAccuracy.target;
    }
  }

  void reload() {
    if (widget.id != 0) {
      setState(() {
        _meal = Meal.get(widget.id);
      });
    }
    _isNew = _meal == null;
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });
    if (_mealForm.currentState!.validate()) {
      Meal meal = Meal(
        id: widget.id,
        value: _valueController.text,
        carbsRatio: double.tryParse(_carbsRatioController.text),
        portionSize: double.tryParse(_portionSizeController.text),
        carbsPerPortion: double.tryParse(_carbsPerPortionController.text),
        delayedBolusDuration:
            int.tryParse(_delayedBolusDurationController.text),
        delayedBolusRate: double.tryParse(_delayedBolusRateController.text),
        notes: _notesController.text,
      );
      meal.mealSource.target = _mealSource;
      meal.mealCategory.target = _mealCategory;
      meal.mealPortionType.target = _mealPortionType;
      meal.portionSizeAccuracy.target = _portionSizeAccuracy;
      meal.carbsRatioAccuracy.target = _carbsRatioAccuracy;

      Meal.put(meal);
      Navigator.pop(context, '${_isNew ? 'New' : ''} Meal Saved');
    }
    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    if (Settings.get().showConfirmationDialogOnCancel &&
        ((_isNew &&
                (_valueController.text != '' ||
                    _mealSource != null ||
                    _mealCategory != null ||
                    _mealPortionType != null ||
                    double.tryParse(_carbsRatioController.text) != null ||
                    double.tryParse(_portionSizeController.text) != null ||
                    double.tryParse(_carbsPerPortionController.text) != null ||
                    _carbsRatioAccuracy != null ||
                    _portionSizeAccuracy != null ||
                    int.tryParse(_delayedBolusDurationController.text) !=
                        null ||
                    double.tryParse(_delayedBolusRateController.text) != null ||
                    _notesController.text != '')) ||
            (!_isNew &&
                (_valueController.text != _meal!.value ||
                    _mealSource != _meal!.mealSource.target ||
                    _mealCategory != _meal!.mealCategory.target ||
                    _mealPortionType != _meal!.mealPortionType.target ||
                    double.tryParse(_carbsRatioController.text) !=
                        _meal!.carbsRatio ||
                    double.tryParse(_portionSizeController.text) !=
                        _meal!.portionSize ||
                    double.tryParse(_carbsPerPortionController.text) !=
                        _meal!.carbsPerPortion ||
                    _carbsRatioAccuracy != _meal!.carbsRatioAccuracy.target ||
                    _portionSizeAccuracy != _meal!.portionSizeAccuracy.target ||
                    int.tryParse(_delayedBolusDurationController.text) !=
                        _meal!.delayedBolusDuration ||
                    double.tryParse(_delayedBolusRateController.text) !=
                        _meal!.delayedBolusRate ||
                    _notesController.text != (_meal!.notes ?? ''))))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: _isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  Future<void> onSelectMealSource(MealSource mealSource) async {
    setState(() {
      _mealSource = mealSource;
      if (mealSource.defaultCarbsRatioAccuracy.hasValue) {
        _carbsRatioAccuracy = mealSource.defaultCarbsRatioAccuracy.target;
      }
      if (mealSource.defaultPortionSizeAccuracy.hasValue) {
        _portionSizeAccuracy = mealSource.defaultPortionSizeAccuracy.target;
      }
      if (mealSource.defaultMealCategory.hasValue) {
        _mealCategory = mealSource.defaultMealCategory.target;
      }
      if (mealSource.defaultMealPortionType.hasValue) {
        _mealPortionType = mealSource.defaultMealPortionType.target;
      }
    });
  }

  void calculateThirdMeasurementOfPortionCarbsRelation(
      {PortionCarbsParameter? parameterToBeCalculated}) {
    double? carbsRatio;
    double? portionSize;
    double? carbsPerPortion;

    if (parameterToBeCalculated != PortionCarbsParameter.carbsRatio &&
        _carbsRatioController.text != '') {
      carbsRatio = double.tryParse(_carbsRatioController.text);
    }
    if (parameterToBeCalculated != PortionCarbsParameter.portionSize &&
        _portionSizeController.text != '') {
      portionSize = double.tryParse(_portionSizeController.text);
    }
    if (parameterToBeCalculated != PortionCarbsParameter.carbsPerPortion &&
        _carbsRatioController.text != '') {
      carbsPerPortion = double.tryParse(_carbsPerPortionController.text);
    }

    if (carbsRatio != null && portionSize != null && carbsPerPortion == null) {
      setState(() {
        _carbsPerPortionController.text =
            Utils.calculateCarbsPerPortion(carbsRatio!, portionSize!)
                .toString();
      });
    }
    if (carbsRatio == null && portionSize != null && carbsPerPortion != null) {
      setState(() {
        _carbsRatioController.text =
            Utils.calculateCarbsRatio(carbsPerPortion!, portionSize!)
                .toString();
      });
    }
    if (carbsRatio != null && portionSize == null && carbsPerPortion != null) {
      setState(() {
        _portionSizeController.text =
            Utils.calculatePortionSize(carbsRatio!, carbsPerPortion!)
                .toString();
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_isNew ? 'New Meal' : _meal!.value),
      ),
      drawer: const Navigation(currentLocation: MealDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            FormWrapper(
              formState: _mealForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                AutoCompleteDropdownButton<MealSource>(
                  selectedItem: _mealSource,
                  label: 'Meal Source',
                  items: _mealSources,
                  onChanged: (value) {
                    if (value != null) {
                      onSelectMealSource(value);
                    }
                  },
                ),
                AutoCompleteDropdownButton<MealCategory>(
                  selectedItem: _mealCategory,
                  label: 'Meal Category',
                  items: _mealCategories,
                  onChanged: (value) {
                    setState(() {
                      _mealCategory = value;
                    });
                  },
                ),
                AutoCompleteDropdownButton<MealPortionType>(
                  selectedItem: _mealPortionType,
                  label: 'Meal Portion Type',
                  items: _mealPortionTypes,
                  onChanged: (value) {
                    setState(() {
                      _mealPortionType = value;
                    });
                  },
                ),
                Row(
                  children: [
                    Expanded(
                      child: TextFormField(
                        decoration: const InputDecoration(
                          labelText: 'Carbs ratio',
                          suffixText: '%',
                        ),
                        controller: _carbsRatioController,
                        keyboardType: const TextInputType.numberWithOptions(
                            decimal: true),
                        onChanged: (_) =>
                            calculateThirdMeasurementOfPortionCarbsRelation(),
                      ),
                    ),
                    IconButton(
                      onPressed: () =>
                          calculateThirdMeasurementOfPortionCarbsRelation(
                              parameterToBeCalculated:
                                  PortionCarbsParameter.carbsRatio),
                      icon: const Icon(Icons.calculate),
                    ),
                  ],
                ),
                Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Expanded(
                      child: TextFormField(
                        decoration: InputDecoration(
                          labelText: 'Portion size',
                          suffixText:
                            nutritionMeasurementSuffixes[Settings.get().nutritionMeasurement.index],
                          alignLabelWithHint: true,
                        ),
                        controller: _portionSizeController,
                        keyboardType: const TextInputType.numberWithOptions(
                            decimal: true),
                        onChanged: (_) =>
                            calculateThirdMeasurementOfPortionCarbsRelation(),
                      ),
                    ),
                    IconButton(
                      onPressed: () =>
                          calculateThirdMeasurementOfPortionCarbsRelation(
                              parameterToBeCalculated:
                                  PortionCarbsParameter.portionSize),
                      icon: const Icon(Icons.calculate),
                    ),
                  ],
                ),
                AutoCompleteDropdownButton<Accuracy>(
                  selectedItem: _portionSizeAccuracy,
                  label: 'Portion Size Accuracy',
                  items: _portionSizeAccuracies,
                  onChanged: (value) {
                    setState(() {
                      _portionSizeAccuracy = value;
                    });
                  },
                ),
                Row(
                  children: [
                    Expanded(
                      child: TextFormField(
                        decoration: InputDecoration(
                          labelText: 'Carbs per portion',
                          suffixText:
                            nutritionMeasurementSuffixes[Settings.get().nutritionMeasurement.index],
                        ),
                        controller: _carbsPerPortionController,
                        keyboardType: const TextInputType.numberWithOptions(
                            decimal: true),
                        onChanged: (_) =>
                            calculateThirdMeasurementOfPortionCarbsRelation(),
                      ),
                    ),
                    IconButton(
                      onPressed: () =>
                          calculateThirdMeasurementOfPortionCarbsRelation(
                              parameterToBeCalculated:
                                  PortionCarbsParameter.carbsPerPortion),
                      icon: const Icon(Icons.calculate),
                    ),
                  ],
                ),
                AutoCompleteDropdownButton<Accuracy>(
                  selectedItem: _carbsRatioAccuracy,
                  label: 'Carbs Ratio Accuracy',
                  items: _carbsRatioAccuracies,
                  onChanged: (value) {
                    setState(() {
                      _carbsRatioAccuracy = value;
                    });
                  },
                ),
                // ignore: todo
                // TODO: display according to time format
                TextFormField(
                  decoration: const InputDecoration(
                    labelText: 'Delayed Bolus Duration',
                    suffixText: ' min',
                  ),
                  controller: _delayedBolusDurationController,
                  keyboardType: const TextInputType.numberWithOptions(),
                ),
                TextFormField(
                  decoration: const InputDecoration(
                    labelText: 'Delayed Bolus Units',
                    suffixText: ' U',
                  ),
                  controller: _delayedBolusRateController,
                  keyboardType:
                      const TextInputType.numberWithOptions(decimal: true),
                ),
                TextFormField(
                  controller: _notesController,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                  keyboardType: TextInputType.multiline,
                ),
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
